@@ -26,25 +26,28 @@
 
 #include <gst/audio/audio.h>
 #include "gstdvdemux.h"
-#include "gstsmptetimecode.h"
 
 /**
  * SECTION:element-dvdemux
  *
+ * <refsect2>
+ * <para>
  * dvdemux splits raw DV into its audio and video components. The audio will be
  * decoded raw samples and the video will be encoded DV video.
- *
- * This element can operate in both push and pull mode depending on the
- * capabilities of the upstream peer.
- *
- * <refsect2>
+ * </para>
+ * <para>
+ * This element can operate in both push and pull mode depending on the capabilities
+ * of the upstream peer.
+ * </para>
  * <title>Example launch line</title>
- * |[
+ * <para>
+ * <programlisting>
  * gst-launch filesrc location=test.dv ! dvdemux name=demux ! queue ! audioconvert ! alsasink demux. ! queue ! dvdec ! xvimagesink
- * ]| This pipeline decodes and renders the raw DV stream to an audio and a videosink.
- * </refsect2>
- *
+ * </programlisting>
+ * This pipeline decodes and renders the raw DV stream to an audio and a videosink.
+ * </para>
  * Last reviewed on 2006-02-27 (0.10.3)
+ * </refsect2>
  */
 
 /* DV output has two modes, normal and wide. The resolution is the same in both
@@ -104,9 +107,15 @@
 #define NTSC_WIDE_PAR_X         40
 #define NTSC_WIDE_PAR_Y         33
 
-GST_DEBUG_CATEGORY_STATIC (dvdemux_debug);
+GST_DEBUG_CATEGORY (dvdemux_debug);
 #define GST_CAT_DEFAULT dvdemux_debug
 
+static const GstElementDetails dvdemux_details =
+GST_ELEMENT_DETAILS ("DV system stream demuxer",
+    "Codec/Demuxer",
+    "Uses libdv to separate DV audio from DV video (libdv.sourceforge.net)",
+    "Erik Walthinsen <omega@cse.ogi.edu>, Wim Taymans <wim@fluendo.com>");
+
 static GstStaticPadTemplate sink_temp = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
@@ -174,16 +183,14 @@ gst_dvdemux_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_static_pad_template (element_class, &sink_temp);
-  gst_element_class_add_static_pad_template (element_class,
-      &video_src_temp);
-  gst_element_class_add_static_pad_template (element_class,
-      &audio_src_temp);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_temp));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&video_src_temp));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&audio_src_temp));
 
-  gst_element_class_set_details_simple (element_class,
-      "DV system stream demuxer", "Codec/Demuxer",
-      "Uses libdv to separate DV audio from DV video (libdv.sourceforge.net)",
-      "Erik Walthinsen <omega@cse.ogi.edu>, Wim Taymans <wim@fluendo.com>");
+  gst_element_class_set_details (element_class, &dvdemux_details);
 
   GST_DEBUG_CATEGORY_INIT (dvdemux_debug, "dvdemux", 0, "DV demuxer element");
 }
@@ -197,10 +204,13 @@ gst_dvdemux_class_init (GstDVDemuxClass * klass)
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
-  gobject_class->finalize = gst_dvdemux_finalize;
+  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_dvdemux_finalize);
 
   gstelement_class->change_state = GST_DEBUG_FUNCPTR (gst_dvdemux_change_state);
   gstelement_class->send_event = GST_DEBUG_FUNCPTR (gst_dvdemux_send_event);
+
+  /* table initialization, only do once */
+  dv_init (0, 0);
 }
 
 static void
@@ -270,7 +280,7 @@ gst_dvdemux_reset (GstDVDemux * dvdemux)
   dvdemux->audio_offset = 0;
   dvdemux->video_offset = 0;
   dvdemux->framecount = 0;
-  g_atomic_int_set (&dvdemux->found_header, 0);
+  gst_atomic_int_set (&dvdemux->found_header, 0);
   dvdemux->frame_len = -1;
   dvdemux->need_segment = FALSE;
   dvdemux->new_media = FALSE;
@@ -284,55 +294,32 @@ gst_dvdemux_reset (GstDVDemux * dvdemux)
   gst_segment_init (&dvdemux->time_segment, GST_FORMAT_TIME);
 }
 
-static GstPad *
-gst_dvdemux_add_pad (GstDVDemux * dvdemux, GstStaticPadTemplate * template)
-{
-  gboolean no_more_pads;
-  GstPad *pad;
-
-  pad = gst_pad_new_from_static_template (template, template->name_template);
-
-  gst_pad_set_query_function (pad, GST_DEBUG_FUNCPTR (gst_dvdemux_src_query));
-
-  gst_pad_set_query_type_function (pad,
-      GST_DEBUG_FUNCPTR (gst_dvdemux_get_src_query_types));
-  gst_pad_set_event_function (pad,
-      GST_DEBUG_FUNCPTR (gst_dvdemux_handle_src_event));
-  gst_pad_use_fixed_caps (pad);
-  gst_pad_set_active (pad, TRUE);
-  gst_element_add_pad (GST_ELEMENT (dvdemux), pad);
-
-  no_more_pads =
-      (dvdemux->videosrcpad != NULL && template == &audio_src_temp) ||
-      (dvdemux->audiosrcpad != NULL && template == &video_src_temp);
-
-  if (no_more_pads)
-    gst_element_no_more_pads (GST_ELEMENT (dvdemux));
-
-  gst_pad_push_event (pad, gst_event_new_new_segment (FALSE,
-          dvdemux->byte_segment.rate, GST_FORMAT_TIME,
-          dvdemux->time_segment.start, dvdemux->time_segment.stop,
-          dvdemux->time_segment.start));
-
-  if (no_more_pads) {
-    gst_element_found_tags (GST_ELEMENT (dvdemux),
-        gst_tag_list_new_full (GST_TAG_CONTAINER_FORMAT, "DV", NULL));
-  }
-
-  return pad;
-}
-
 static void
-gst_dvdemux_remove_pads (GstDVDemux * dvdemux)
+gst_dvdemux_add_pads (GstDVDemux * dvdemux)
 {
-  if (dvdemux->videosrcpad) {
-    gst_element_remove_pad (GST_ELEMENT (dvdemux), dvdemux->videosrcpad);
-    dvdemux->videosrcpad = NULL;
-  }
-  if (dvdemux->audiosrcpad) {
-    gst_element_remove_pad (GST_ELEMENT (dvdemux), dvdemux->audiosrcpad);
-    dvdemux->audiosrcpad = NULL;
-  }
+  dvdemux->videosrcpad =
+      gst_pad_new_from_static_template (&video_src_temp, "video");
+  gst_pad_set_query_function (dvdemux->videosrcpad,
+      GST_DEBUG_FUNCPTR (gst_dvdemux_src_query));
+  gst_pad_set_query_type_function (dvdemux->videosrcpad,
+      GST_DEBUG_FUNCPTR (gst_dvdemux_get_src_query_types));
+  gst_pad_set_event_function (dvdemux->videosrcpad,
+      GST_DEBUG_FUNCPTR (gst_dvdemux_handle_src_event));
+  gst_pad_use_fixed_caps (dvdemux->videosrcpad);
+  gst_element_add_pad (GST_ELEMENT (dvdemux), dvdemux->videosrcpad);
+
+  dvdemux->audiosrcpad =
+      gst_pad_new_from_static_template (&audio_src_temp, "audio");
+  gst_pad_set_query_function (dvdemux->audiosrcpad,
+      GST_DEBUG_FUNCPTR (gst_dvdemux_src_query));
+  gst_pad_set_query_type_function (dvdemux->audiosrcpad,
+      GST_DEBUG_FUNCPTR (gst_dvdemux_get_src_query_types));
+  gst_pad_set_event_function (dvdemux->audiosrcpad,
+      GST_DEBUG_FUNCPTR (gst_dvdemux_handle_src_event));
+  gst_pad_use_fixed_caps (dvdemux->audiosrcpad);
+  gst_element_add_pad (GST_ELEMENT (dvdemux), dvdemux->audiosrcpad);
+
+  gst_element_no_more_pads (GST_ELEMENT (dvdemux));
 }
 
 static gboolean
@@ -353,8 +340,7 @@ gst_dvdemux_src_convert (GstDVDemux * dvdemux, GstPad * pad,
   if (dvdemux->decoder == NULL)
     goto error;
 
-  GST_INFO_OBJECT (pad,
-      "src_value:%" G_GINT64_FORMAT ", src_format:%d, dest_format:%d",
+  GST_INFO_OBJECT (pad, "src_value:%lld, src_format:%d, dest_format:%d",
       src_value, src_format, *dest_format);
 
   switch (src_format) {
@@ -439,8 +425,7 @@ gst_dvdemux_src_convert (GstDVDemux * dvdemux, GstPad * pad,
   }
 
 done:
-  GST_INFO_OBJECT (pad,
-      "Result : dest_format:%d, dest_value:%" G_GINT64_FORMAT ", res:%d",
+  GST_INFO_OBJECT (pad, "Result : dest_format:%d, dest_value:%lld, res:%d",
       *dest_format, *dest_value, res);
   return res;
 
@@ -459,8 +444,7 @@ gst_dvdemux_sink_convert (GstDVDemux * dvdemux, GstFormat src_format,
   gboolean res = TRUE;
 
   GST_DEBUG_OBJECT (dvdemux, "%d -> %d", src_format, *dest_format);
-  GST_INFO_OBJECT (dvdemux,
-      "src_value:%" G_GINT64_FORMAT ", src_format:%d, dest_format:%d",
+  GST_INFO_OBJECT (dvdemux, "src_value:%lld, src_format:%d, dest_format:%d",
       src_value, src_format, *dest_format);
 
   if (*dest_format == GST_FORMAT_DEFAULT)
@@ -516,8 +500,7 @@ gst_dvdemux_sink_convert (GstDVDemux * dvdemux, GstFormat src_format,
     default:
       res = FALSE;
   }
-  GST_INFO_OBJECT (dvdemux,
-      "Result : dest_format:%d, dest_value:%" G_GINT64_FORMAT ", res:%d",
+  GST_INFO_OBJECT (dvdemux, "Result : dest_format:%d, dest_value:%lld, res:%d",
       *dest_format, *dest_value, res);
 
 done:
@@ -573,23 +556,26 @@ gst_dvdemux_src_query (GstPad * pad, GstQuery * query)
       GstFormat format;
       GstFormat format2;
       gint64 end;
+      GstPad *peer;
 
-      /* First ask the peer in the original format */
-      if (!gst_pad_peer_query (dvdemux->sinkpad, query)) {
-        /* get target format */
-        gst_query_parse_duration (query, &format, NULL);
+      /* get target format */
+      gst_query_parse_duration (query, &format, NULL);
 
-        /* change query to bytes to perform on peer */
-        gst_query_set_duration (query, GST_FORMAT_BYTES, -1);
+      /* change query to bytes to perform on peer */
+      gst_query_set_duration (query, GST_FORMAT_BYTES, -1);
 
-        /* Now ask the peer in BYTES format and try to convert */
-        if (!gst_pad_peer_query (dvdemux->sinkpad, query)) {
+      if ((peer = gst_pad_get_peer (dvdemux->sinkpad))) {
+        /* ask peer for total length */
+        if (!(res = gst_pad_query (peer, query))) {
+          gst_object_unref (peer);
           goto error;
         }
 
         /* get peer total length */
         gst_query_parse_duration (query, NULL, &end);
 
+        gst_object_unref (peer);
+
         /* convert end to requested format */
         if (end != -1) {
           format2 = format;
@@ -597,9 +583,11 @@ gst_dvdemux_src_query (GstPad * pad, GstQuery * query)
                       GST_FORMAT_BYTES, end, &format2, &end))) {
             goto error;
           }
-          gst_query_set_duration (query, format, end);
         }
+      } else {
+        end = -1;
       }
+      gst_query_set_duration (query, format, end);
       break;
     }
     case GST_QUERY_CONVERT:
@@ -802,7 +790,7 @@ gst_dvdemux_convert_src_pair (GstDVDemux * dvdemux, GstPad * pad,
               src_format, src_start, &dst_format, dst_start))) {
     goto done;
   }
-  GST_INFO ("Finished conversion of start: %" G_GINT64_FORMAT, *dst_start);
+  GST_INFO ("Finished conversion of start: %lld", *dst_start);
 
   GST_INFO ("starting conversion of stop");
   /* bring the format to time on srcpad. */
@@ -811,7 +799,7 @@ gst_dvdemux_convert_src_pair (GstDVDemux * dvdemux, GstPad * pad,
     /* could not convert seek format to time offset */
     goto done;
   }
-  GST_INFO ("Finished conversion of stop: %" G_GINT64_FORMAT, *dst_stop);
+  GST_INFO ("Finished conversion of stop: %lld", *dst_stop);
 done:
   return res;
 }
@@ -830,7 +818,7 @@ gst_dvdemux_convert_sink_pair (GstDVDemux * dvdemux,
               src_format, src_start, &dst_format, dst_start))) {
     goto done;
   }
-  GST_INFO ("Finished conversion of start: %" G_GINT64_FORMAT, *dst_start);
+  GST_INFO ("Finished conversion of start: %lld", *dst_start);
 
   GST_INFO ("starting conversion of stop");
   /* bring the format to time on srcpad. */
@@ -839,7 +827,7 @@ gst_dvdemux_convert_sink_pair (GstDVDemux * dvdemux,
     /* could not convert seek format to time offset */
     goto done;
   }
-  GST_INFO ("Finished conversion of stop: %" G_GINT64_FORMAT, *dst_stop);
+  GST_INFO ("Finished conversion of stop: %lld", *dst_stop);
 done:
   return res;
 }
@@ -889,13 +877,17 @@ gst_dvdemux_convert_segment (GstDVDemux * dvdemux, GstSegment * src,
  *
  * Convert the time seek to a bytes seek and send it
  * upstream
+ *
+ * FIXME, upstream might be able to perform time based
+ * seek too.
+ *
  * Does not take ownership of the event.
  */
 static gboolean
 gst_dvdemux_handle_push_seek (GstDVDemux * dvdemux, GstPad * pad,
     GstEvent * event)
 {
-  gboolean res = FALSE;
+  gboolean res;
   gdouble rate;
   GstSeekFlags flags;
   GstFormat format;
@@ -907,24 +899,19 @@ gst_dvdemux_handle_push_seek (GstDVDemux * dvdemux, GstPad * pad,
   gst_event_parse_seek (event, &rate, &format, &flags,
       &cur_type, &cur, &stop_type, &stop);
 
-  /* First try if upstream can handle time based seeks */
-  if (format == GST_FORMAT_TIME)
-    res = gst_pad_push_event (dvdemux->sinkpad, gst_event_ref (event));
+  /* we convert the start/stop on the srcpad to the byte format
+   * on the sinkpad and forward the event */
+  res = gst_dvdemux_convert_src_to_sink (dvdemux, pad,
+      format, cur, stop, GST_FORMAT_BYTES, &start_position, &end_position);
+  if (!res)
+    goto done;
 
-  if (!res) {
-    /* we convert the start/stop on the srcpad to the byte format
-     * on the sinkpad and forward the event */
-    res = gst_dvdemux_convert_src_to_sink (dvdemux, pad,
-        format, cur, stop, GST_FORMAT_BYTES, &start_position, &end_position);
-    if (!res)
-      goto done;
+  /* now this is the updated seek event on bytes */
+  newevent = gst_event_new_seek (rate, GST_FORMAT_BYTES, flags,
+      cur_type, start_position, stop_type, end_position);
 
-    /* now this is the updated seek event on bytes */
-    newevent = gst_event_new_seek (rate, GST_FORMAT_BYTES, flags,
-        cur_type, start_position, stop_type, end_position);
+  res = gst_pad_push_event (dvdemux->sinkpad, newevent);
 
-    res = gst_pad_push_event (dvdemux->sinkpad, newevent);
-  }
 done:
   return res;
 }
@@ -1013,7 +1000,7 @@ gst_dvdemux_handle_pull_seek (GstDVDemux * demux, GstPad * pad,
 
     /* convert input format to TIME */
     conv = GST_FORMAT_TIME;
-    if (!(gst_dvdemux_convert_src_pair (demux, pad,
+    if (!(res = gst_dvdemux_convert_src_pair (demux, pad,
                 format, cur, stop, conv, &cur, &stop)))
       goto no_format;
 
@@ -1084,17 +1071,15 @@ gst_dvdemux_handle_pull_seek (GstDVDemux * demux, GstPad * pad,
     if ((stop = demux->time_segment.stop) == -1)
       stop = demux->time_segment.duration;
 
-    GST_INFO_OBJECT (demux,
-        "Saving newsegment event to be sent in streaming thread");
+    /* now send the newsegment */
+    GST_DEBUG_OBJECT (demux, "Sending newsegment from %" GST_TIME_FORMAT
+        " to %" GST_TIME_FORMAT, GST_TIME_ARGS (demux->time_segment.start),
+        GST_TIME_ARGS (stop));
 
-    if (demux->pending_segment)
-      gst_event_unref (demux->pending_segment);
-
-    demux->pending_segment = gst_event_new_new_segment (FALSE,
-        demux->time_segment.rate, demux->time_segment.format,
-        demux->time_segment.last_stop, stop, demux->time_segment.time);
-
-    demux->need_segment = FALSE;
+    gst_dvdemux_push_event (demux,
+        gst_event_new_new_segment (FALSE,
+            demux->time_segment.rate, demux->time_segment.format,
+            demux->time_segment.last_stop, stop, demux->time_segment.time));
   }
 
   demux->running = TRUE;
@@ -1147,15 +1132,12 @@ gst_dvdemux_send_event (GstElement * element, GstEvent * event)
       } else {
         GST_OBJECT_UNLOCK (dvdemux);
 
-        if (dvdemux->seek_handler) {
+        if (dvdemux->seek_handler)
           res = dvdemux->seek_handler (dvdemux, dvdemux->videosrcpad, event);
-          gst_event_unref (event);
-        }
       }
       break;
     }
     default:
-      res = GST_ELEMENT_CLASS (parent_class)->send_event (element, event);
       break;
   }
 
@@ -1181,19 +1163,13 @@ gst_dvdemux_handle_src_event (GstPad * pad, GstEvent * event)
       break;
     case GST_EVENT_QOS:
       /* we can't really (yet) do QoS */
-      res = FALSE;
-      break;
     case GST_EVENT_NAVIGATION:
       /* no navigation either... */
+    default:
       res = FALSE;
       break;
-    default:
-      res = gst_pad_push_event (dvdemux->sinkpad, event);
-      event = NULL;
-      break;
   }
-  if (event)
-    gst_event_unref (event);
+  gst_event_unref (event);
 
   gst_object_unref (dvdemux);
 
@@ -1206,44 +1182,40 @@ gst_dvdemux_demux_audio (GstDVDemux * dvdemux, GstBuffer * buffer,
     guint64 duration)
 {
   gint num_samples;
+  gint frequency, channels;
   GstFlowReturn ret;
   const guint8 *data;
 
+  frequency = dv_get_frequency (dvdemux->decoder);
+  channels = dv_get_num_channels (dvdemux->decoder);
+
   data = GST_BUFFER_DATA (buffer);
 
+  /* check if format changed */
+  if ((frequency != dvdemux->frequency) || (channels != dvdemux->channels)) {
+    GstCaps *caps;
+
+    dvdemux->frequency = frequency;
+    dvdemux->channels = channels;
+
+    /* and set new caps */
+    caps = gst_caps_new_simple ("audio/x-raw-int",
+        "rate", G_TYPE_INT, frequency,
+        "depth", G_TYPE_INT, 16,
+        "width", G_TYPE_INT, 16,
+        "signed", G_TYPE_BOOLEAN, TRUE,
+        "channels", G_TYPE_INT, channels,
+        "endianness", G_TYPE_INT, G_BYTE_ORDER, NULL);
+    gst_pad_set_caps (dvdemux->audiosrcpad, caps);
+    gst_caps_unref (caps);
+  }
+
   dv_decode_full_audio (dvdemux->decoder, data, dvdemux->audio_buffers);
 
-  if (G_LIKELY ((num_samples = dv_get_num_samples (dvdemux->decoder)) > 0)) {
+  if ((num_samples = dv_get_num_samples (dvdemux->decoder)) > 0) {
     gint16 *a_ptr;
     gint i, j;
     GstBuffer *outbuf;
-    gint frequency, channels;
-
-    if (G_UNLIKELY (dvdemux->audiosrcpad == NULL))
-      dvdemux->audiosrcpad = gst_dvdemux_add_pad (dvdemux, &audio_src_temp);
-
-    /* get initial format or check if format changed */
-    frequency = dv_get_frequency (dvdemux->decoder);
-    channels = dv_get_num_channels (dvdemux->decoder);
-
-    if (G_UNLIKELY ((frequency != dvdemux->frequency)
-            || (channels != dvdemux->channels))) {
-      GstCaps *caps;
-
-      dvdemux->frequency = frequency;
-      dvdemux->channels = channels;
-
-      /* and set new caps */
-      caps = gst_caps_new_simple ("audio/x-raw-int",
-          "rate", G_TYPE_INT, frequency,
-          "depth", G_TYPE_INT, 16,
-          "width", G_TYPE_INT, 16,
-          "signed", G_TYPE_BOOLEAN, TRUE,
-          "channels", G_TYPE_INT, channels,
-          "endianness", G_TYPE_INT, G_BYTE_ORDER, NULL);
-      gst_pad_set_caps (dvdemux->audiosrcpad, caps);
-      gst_caps_unref (caps);
-    }
 
     outbuf = gst_buffer_new_and_alloc (num_samples *
         sizeof (gint16) * dvdemux->channels);
@@ -1265,8 +1237,6 @@ gst_dvdemux_demux_audio (GstDVDemux * dvdemux, GstBuffer * buffer,
     dvdemux->audio_offset += num_samples;
     GST_BUFFER_OFFSET_END (outbuf) = dvdemux->audio_offset;
 
-    if (dvdemux->new_media)
-      GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DISCONT);
     gst_buffer_set_caps (outbuf, GST_PAD_CAPS (dvdemux->audiosrcpad));
 
     ret = gst_pad_push (dvdemux->audiosrcpad, outbuf);
@@ -1288,12 +1258,9 @@ gst_dvdemux_demux_video (GstDVDemux * dvdemux, GstBuffer * buffer,
   gboolean wide;
   GstFlowReturn ret = GST_FLOW_OK;
 
-  if (G_UNLIKELY (dvdemux->videosrcpad == NULL))
-    dvdemux->videosrcpad = gst_dvdemux_add_pad (dvdemux, &video_src_temp);
-
   /* get params */
   /* framerate is already up-to-date */
-  height = dvdemux->decoder->height;
+  height = (dvdemux->PAL ? PAL_HEIGHT : NTSC_HEIGHT);
   wide = dv_format_wide (dvdemux->decoder);
 
   /* see if anything changed */
@@ -1304,7 +1271,7 @@ gst_dvdemux_demux_video (GstDVDemux * dvdemux, GstBuffer * buffer,
     dvdemux->height = height;
     dvdemux->wide = wide;
 
-    if (dvdemux->decoder->system == e_dv_system_625_50) {
+    if (dvdemux->PAL) {
       if (wide) {
         par_x = PAL_WIDE_PAR_X;
         par_y = PAL_WIDE_PAR_Y;
@@ -1342,8 +1309,6 @@ gst_dvdemux_demux_video (GstDVDemux * dvdemux, GstBuffer * buffer,
   GST_BUFFER_OFFSET_END (outbuf) = dvdemux->video_offset + 1;
   GST_BUFFER_DURATION (outbuf) = duration;
 
-  if (dvdemux->new_media)
-    GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DISCONT);
   gst_buffer_set_caps (outbuf, GST_PAD_CAPS (dvdemux->videosrcpad));
 
   GST_DEBUG ("pushing video %" GST_TIME_FORMAT,
@@ -1356,73 +1321,6 @@ gst_dvdemux_demux_video (GstDVDemux * dvdemux, GstBuffer * buffer,
   return ret;
 }
 
-static int
-get_ssyb_offset (int dif, int ssyb)
-{
-  int offset;
-
-  offset = dif * 12000;         /* to dif */
-  offset += 80 * (1 + (ssyb / 6));      /* to subcode pack */
-  offset += 3;                  /* past header */
-  offset += 8 * (ssyb % 6);     /* to ssyb */
-
-  return offset;
-}
-
-static gboolean
-gst_dvdemux_get_timecode (GstDVDemux * dvdemux, GstBuffer * buffer,
-    GstSMPTETimeCode * timecode)
-{
-  guint8 *data = GST_BUFFER_DATA (buffer);
-  int offset;
-  int dif;
-  int n_difs = dvdemux->decoder->num_dif_seqs;
-
-  for (dif = 0; dif < n_difs; dif++) {
-    offset = get_ssyb_offset (dif, 3);
-    if (data[offset + 3] == 0x13) {
-      timecode->frames = ((data[offset + 4] >> 4) & 0x3) * 10 +
-          (data[offset + 4] & 0xf);
-      timecode->seconds = ((data[offset + 5] >> 4) & 0x3) * 10 +
-          (data[offset + 5] & 0xf);
-      timecode->minutes = ((data[offset + 6] >> 4) & 0x3) * 10 +
-          (data[offset + 6] & 0xf);
-      timecode->hours = ((data[offset + 7] >> 4) & 0x3) * 10 +
-          (data[offset + 7] & 0xf);
-      GST_DEBUG ("got timecode %" GST_SMPTE_TIME_CODE_FORMAT,
-          GST_SMPTE_TIME_CODE_ARGS (timecode));
-      return TRUE;
-    }
-  }
-
-  return FALSE;
-}
-
-static gboolean
-gst_dvdemux_is_new_media (GstDVDemux * dvdemux, GstBuffer * buffer)
-{
-  guint8 *data = GST_BUFFER_DATA (buffer);
-  int aaux_offset;
-  int dif;
-  int n_difs;
-
-  n_difs = dvdemux->decoder->num_dif_seqs;
-
-  for (dif = 0; dif < n_difs; dif++) {
-    if (dif & 1) {
-      aaux_offset = (dif * 12000) + (6 + 16 * 1) * 80 + 3;
-    } else {
-      aaux_offset = (dif * 12000) + (6 + 16 * 4) * 80 + 3;
-    }
-    if (data[aaux_offset + 0] == 0x51) {
-      if ((data[aaux_offset + 2] & 0x80) == 0)
-        return TRUE;
-    }
-  }
-
-  return FALSE;
-}
-
 /* takes ownership of buffer */
 static GstFlowReturn
 gst_dvdemux_demux_frame (GstDVDemux * dvdemux, GstBuffer * buffer)
@@ -1431,16 +1329,15 @@ gst_dvdemux_demux_frame (GstDVDemux * dvdemux, GstBuffer * buffer)
   GstFlowReturn aret, vret, ret;
   guint8 *data;
   guint64 duration;
-  GstSMPTETimeCode timecode;
-  int frame_number;
 
   if (G_UNLIKELY (dvdemux->need_segment)) {
     GstEvent *event;
     GstFormat format;
+    gboolean res;
 
     /* convert to time and store as start/end_timestamp */
     format = GST_FORMAT_TIME;
-    if (!(gst_dvdemux_convert_sink_pair (dvdemux,
+    if (!(res = gst_dvdemux_convert_sink_pair (dvdemux,
                 GST_FORMAT_BYTES, dvdemux->byte_segment.start,
                 dvdemux->byte_segment.stop, format,
                 &dvdemux->time_segment.start, &dvdemux->time_segment.stop)))
@@ -1452,7 +1349,7 @@ gst_dvdemux_demux_frame (GstDVDemux * dvdemux, GstBuffer * buffer)
 
     /* calculate current frame number */
     format = GST_FORMAT_DEFAULT;
-    if (!(gst_dvdemux_src_convert (dvdemux, dvdemux->videosrcpad,
+    if (!(res = gst_dvdemux_src_convert (dvdemux, dvdemux->videosrcpad,
                 GST_FORMAT_TIME, dvdemux->time_segment.start,
                 &format, &dvdemux->frame_offset)))
       goto segment_error;
@@ -1471,12 +1368,6 @@ gst_dvdemux_demux_frame (GstDVDemux * dvdemux, GstBuffer * buffer)
     dvdemux->need_segment = FALSE;
   }
 
-  gst_dvdemux_get_timecode (dvdemux, buffer, &timecode);
-  gst_smpte_time_code_get_frame_number (
-      (dvdemux->decoder->system == e_dv_system_625_50) ?
-      GST_SMPTE_TIME_CODE_SYSTEM_25 : GST_SMPTE_TIME_CODE_SYSTEM_30,
-      &frame_number, &timecode);
-
   next_ts = gst_util_uint64_scale_int (
       (dvdemux->frame_offset + 1) * GST_SECOND,
       dvdemux->framerate_denominator, dvdemux->framerate_numerator);
@@ -1485,13 +1376,8 @@ gst_dvdemux_demux_frame (GstDVDemux * dvdemux, GstBuffer * buffer)
   data = GST_BUFFER_DATA (buffer);
 
   dv_parse_packs (dvdemux->decoder, data);
-  dvdemux->new_media = FALSE;
-  if (gst_dvdemux_is_new_media (dvdemux, buffer) &&
-      dvdemux->frames_since_new_media > 2) {
+  if (G_UNLIKELY (dv_is_new_recording (dvdemux->decoder, data)))
     dvdemux->new_media = TRUE;
-    dvdemux->frames_since_new_media = 0;
-  }
-  dvdemux->frames_since_new_media++;
 
   /* does not take ownership of buffer */
   aret = ret = gst_dvdemux_demux_audio (dvdemux, buffer, duration);
@@ -1511,15 +1397,10 @@ gst_dvdemux_demux_frame (GstDVDemux * dvdemux, GstBuffer * buffer)
     goto done;
   }
 
+  ret = GST_FLOW_OK;
   gst_segment_set_last_stop (&dvdemux->time_segment, GST_FORMAT_TIME, next_ts);
   dvdemux->frame_offset++;
 
-  /* check for the end of the segment */
-  if (dvdemux->time_segment.stop != -1 && next_ts > dvdemux->time_segment.stop)
-    ret = GST_FLOW_UNEXPECTED;
-  else
-    ret = GST_FLOW_OK;
-
 done:
   return ret;
 
@@ -1550,15 +1431,16 @@ gst_dvdemux_flush (GstDVDemux * dvdemux)
       goto parse_header_error;
 
     /* after parsing the header we know the length of the data */
-    length = dvdemux->frame_len = dvdemux->decoder->frame_size;
-    if (dvdemux->decoder->system == e_dv_system_625_50) {
+    dvdemux->PAL = dv_system_50_fields (dvdemux->decoder);
+    length = dvdemux->frame_len = (dvdemux->PAL ? PAL_BUFFER : NTSC_BUFFER);
+    if (dvdemux->PAL) {
       dvdemux->framerate_numerator = PAL_FRAMERATE_NUMERATOR;
       dvdemux->framerate_denominator = PAL_FRAMERATE_DENOMINATOR;
     } else {
       dvdemux->framerate_numerator = NTSC_FRAMERATE_NUMERATOR;
       dvdemux->framerate_denominator = NTSC_FRAMERATE_DENOMINATOR;
     }
-    g_atomic_int_set (&dvdemux->found_header, 1);
+    gst_atomic_int_set (&dvdemux->found_header, 1);
 
     /* let demux_video set the height, it needs to detect when things change so
      * it can reset caps */
@@ -1602,7 +1484,6 @@ gst_dvdemux_chain (GstPad * pad, GstBuffer * buffer)
 {
   GstDVDemux *dvdemux;
   GstFlowReturn ret;
-  GstClockTime timestamp;
 
   dvdemux = GST_DVDEMUX (gst_pad_get_parent (pad));
 
@@ -1612,13 +1493,9 @@ gst_dvdemux_chain (GstPad * pad, GstBuffer * buffer)
   if (G_UNLIKELY (GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_DISCONT)))
     gst_adapter_clear (dvdemux->adapter);
 
-  /* a timestamp always should be respected */
-  timestamp = GST_BUFFER_TIMESTAMP (buffer);
-  if (GST_CLOCK_TIME_IS_VALID (timestamp)) {
-    gst_segment_set_last_stop (&dvdemux->time_segment, GST_FORMAT_TIME,
-        timestamp);
-    /* FIXME, adjust frame_offset and other counters */
-  }
+  /* temporary hack? Can't do this from the state change */
+  if (G_UNLIKELY (!dvdemux->videosrcpad))
+    gst_dvdemux_add_pads (dvdemux);
 
   gst_adapter_push (dvdemux->adapter, buffer);
 
@@ -1656,6 +1533,10 @@ gst_dvdemux_loop (GstPad * pad)
   dvdemux = GST_DVDEMUX (gst_pad_get_parent (pad));
 
   if (G_UNLIKELY (g_atomic_int_get (&dvdemux->found_header) == 0)) {
+    /* add pads.. why is this again? */
+    if (!dvdemux->videosrcpad)
+      gst_dvdemux_add_pads (dvdemux);
+
     GST_DEBUG_OBJECT (dvdemux, "pulling first buffer");
     /* pull in NTSC sized buffer to figure out the frame
      * length */
@@ -1666,7 +1547,7 @@ gst_dvdemux_loop (GstPad * pad)
 
     /* check buffer size, don't want to read small buffers */
     if (G_UNLIKELY (GST_BUFFER_SIZE (buffer) < NTSC_BUFFER))
-      goto small_buffer;
+      goto pause;
 
     data = GST_BUFFER_DATA (buffer);
 
@@ -1675,8 +1556,9 @@ gst_dvdemux_loop (GstPad * pad)
       goto parse_header_error;
 
     /* after parsing the header we know the length of the data */
-    dvdemux->frame_len = dvdemux->decoder->frame_size;
-    if (dvdemux->decoder->system == e_dv_system_625_50) {
+    dvdemux->PAL = dv_system_50_fields (dvdemux->decoder);
+    dvdemux->frame_len = (dvdemux->PAL ? PAL_BUFFER : NTSC_BUFFER);
+    if (dvdemux->PAL) {
       dvdemux->framerate_numerator = PAL_FRAMERATE_NUMERATOR;
       dvdemux->framerate_denominator = PAL_FRAMERATE_DENOMINATOR;
     } else {
@@ -1697,7 +1579,7 @@ gst_dvdemux_loop (GstPad * pad)
       /* setting header and prrforming the seek must be atomic */
       GST_OBJECT_LOCK (dvdemux);
       /* got header now */
-      g_atomic_int_set (&dvdemux->found_header, 1);
+      gst_atomic_int_set (&dvdemux->found_header, 1);
 
       /* now perform pending seek if any. */
       event = dvdemux->seek_event;
@@ -1721,29 +1603,21 @@ gst_dvdemux_loop (GstPad * pad)
       }
     }
   }
-
-
-  if (G_UNLIKELY (dvdemux->pending_segment)) {
-
-    /* now send the newsegment */
-    GST_DEBUG_OBJECT (dvdemux, "Sending newsegment from");
-
-    gst_dvdemux_push_event (dvdemux, dvdemux->pending_segment);
-    dvdemux->pending_segment = NULL;
-  }
-
   if (G_LIKELY (buffer == NULL)) {
     GST_DEBUG_OBJECT (dvdemux, "pulling buffer at offset %" G_GINT64_FORMAT,
         dvdemux->byte_segment.last_stop);
 
     ret = gst_pad_pull_range (dvdemux->sinkpad,
         dvdemux->byte_segment.last_stop, dvdemux->frame_len, &buffer);
+    if (ret == GST_FLOW_UNEXPECTED)
+      goto eos;
+
     if (ret != GST_FLOW_OK)
       goto pause;
 
     /* check buffer size, don't want to read small buffers */
     if (GST_BUFFER_SIZE (buffer) < dvdemux->frame_len)
-      goto small_buffer;
+      goto pause;
   }
   /* and decode the buffer */
   ret = gst_dvdemux_demux_frame (dvdemux, buffer);
@@ -1753,7 +1627,6 @@ gst_dvdemux_loop (GstPad * pad)
   /* and position ourselves for the next buffer */
   dvdemux->byte_segment.last_stop += dvdemux->frame_len;
 
-done:
   gst_object_unref (dvdemux);
 
   return;
@@ -1764,43 +1637,30 @@ parse_header_error:
     GST_ELEMENT_ERROR (dvdemux, STREAM, DECODE,
         (NULL), ("Error parsing DV header"));
     gst_buffer_unref (buffer);
-    dvdemux->running = FALSE;
-    gst_pad_pause_task (dvdemux->sinkpad);
-    gst_dvdemux_push_event (dvdemux, gst_event_new_eos ());
-    goto done;
+    ret = GST_FLOW_ERROR;
+    /* will stop the task */
+    goto pause;
   }
-small_buffer:
+eos:
   {
-    GST_ELEMENT_ERROR (dvdemux, STREAM, DECODE,
-        (NULL), ("Error reading buffer"));
-    gst_buffer_unref (buffer);
-    dvdemux->running = FALSE;
-    gst_pad_pause_task (dvdemux->sinkpad);
+    GST_LOG_OBJECT (dvdemux, "got eos");
     gst_dvdemux_push_event (dvdemux, gst_event_new_eos ());
-    goto done;
+    /* pause with non-fatal error */
+    ret = GST_FLOW_WRONG_STATE;
+    goto pause;
   }
 pause:
   {
-    GST_INFO_OBJECT (dvdemux, "pausing task, %s", gst_flow_get_name (ret));
+    GST_LOG_OBJECT (dvdemux, "pausing task %d", ret);
     dvdemux->running = FALSE;
     gst_pad_pause_task (dvdemux->sinkpad);
-    if (ret == GST_FLOW_UNEXPECTED) {
-      GST_LOG_OBJECT (dvdemux, "got eos");
-      /* perform EOS logic */
-      if (dvdemux->time_segment.flags & GST_SEEK_FLAG_SEGMENT) {
-        gst_element_post_message (GST_ELEMENT (dvdemux),
-            gst_message_new_segment_done (GST_OBJECT_CAST (dvdemux),
-                dvdemux->time_segment.format, dvdemux->time_segment.last_stop));
-      } else {
-        gst_dvdemux_push_event (dvdemux, gst_event_new_eos ());
-      }
-    } else if (ret == GST_FLOW_NOT_LINKED || ret < GST_FLOW_UNEXPECTED) {
-      /* for fatal errors or not-linked we post an error message */
+    if (GST_FLOW_IS_FATAL (ret)) {
+      /* for fatal errors we post an error message */
       GST_ELEMENT_ERROR (dvdemux, STREAM, FAILED,
           (NULL), ("streaming stopped, reason %s", gst_flow_get_name (ret)));
       gst_dvdemux_push_event (dvdemux, gst_event_new_eos ());
     }
-    goto done;
+    gst_object_unref (dvdemux);
   }
 }
 
@@ -1882,8 +1742,6 @@ gst_dvdemux_change_state (GstElement * element, GstStateChange transition)
       gst_adapter_clear (dvdemux->adapter);
       dv_decoder_free (dvdemux->decoder);
       dvdemux->decoder = NULL;
-
-      gst_dvdemux_remove_pads (dvdemux);
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
     {
@@ -1891,9 +1749,6 @@ gst_dvdemux_change_state (GstElement * element, GstStateChange transition)
 
       event_p = &dvdemux->seek_event;
       gst_event_replace (event_p, NULL);
-      if (dvdemux->pending_segment)
-        gst_event_unref (dvdemux->pending_segment);
-      dvdemux->pending_segment = NULL;
       break;
     }
     default:
