@@ -31,12 +31,11 @@
 #include "flac_compat.h"
 
 
-GstElementDetails flacenc_details = {
-  "FLAC encoder",
-  "Codec/Encoder/Audio",
-  "Encodes audio with the FLAC lossless audio encoder",
-  "Wim Taymans <wim.taymans@chello.be>",
-};
+static const GstElementDetails flacenc_details =
+GST_ELEMENT_DETAILS ("FLAC audio encoder",
+    "Codec/Encoder/Audio",
+    "Encodes audio with the FLAC lossless audio encoder",
+    "Wim Taymans <wim.taymans@chello.be>");
 
 #define FLAC_SINK_CAPS \
   "audio/x-raw-int, "               \
@@ -44,7 +43,7 @@ GstElementDetails flacenc_details = {
   "signed = (boolean) TRUE, "       \
   "width = (int) 16, "              \
   "depth = (int) 16, "              \
-  "rate = (int) [ 11025, 48000 ], " \
+  "rate = (int) [ 8000, 48000 ], " \
   "channels = (int) [ 1, 2 ]"
 
 static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
