@@ -1,5 +1,6 @@
 /* GStreamer
  * Copyright (C) <2002> Thomas Vander Stichele <thomas@apestaart.org>
+ * Copyright (C) <2006> Jürg Billeter <j@bitron.ch>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -34,6 +35,13 @@ G_BEGIN_DECLS
 #define GST_GCONF_VIDEOSRC_KEY  "default/videosrc"
 #define GST_GCONF_VIDEOSINK_KEY "default/videosink"
 
+enum
+{
+  GCONF_PROFILE_SOUNDS,
+  GCONF_PROFILE_MUSIC,
+  GCONF_PROFILE_CHAT
+};
+
 gchar *         gst_gconf_get_string            (const gchar *key);
 void            gst_gconf_set_string            (const gchar *key, 
                                                  const gchar *value);
@@ -41,7 +49,7 @@ void            gst_gconf_set_string            (const gchar *key,
 GstElement *    gst_gconf_render_bin_from_key           (const gchar *key);
 
 GstElement *    gst_gconf_get_default_video_sink (void);
-GstElement *    gst_gconf_get_default_audio_sink (void);
+GstElement *    gst_gconf_get_default_audio_sink (int profile);
 GstElement *    gst_gconf_get_default_video_src (void);
 GstElement *    gst_gconf_get_default_audio_src (void);
 GstElement *    gst_gconf_get_default_visualization_element (void);
