@@ -40,12 +40,11 @@ static void
 gst_gconf_audio_src_base_init (gpointer klass)
 {
   GstElementClass *eklass = GST_ELEMENT_CLASS (klass);
-  GstElementDetails gst_gconf_audio_src_details = {
-    "GConf audio source",
-    "Source/Audio",
-    "Audio source embedding the GConf-settings for audio input",
-    "Ronald Bultje <rbultje@ronald.bitfreak.net>"
-  };
+  static const GstElementDetails gst_gconf_audio_src_details =
+      GST_ELEMENT_DETAILS ("GConf audio source",
+      "Source/Audio",
+      "Audio source embedding the GConf-settings for audio input",
+      "Ronald Bultje <rbultje@ronald.bitfreak.net>");
   GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
       GST_PAD_SRC,
       GST_PAD_ALWAYS,
@@ -103,7 +102,7 @@ gst_gconf_audio_src_init (GstGConfAudioSrc * src,
   src->client = gconf_client_get_default ();
   gconf_client_add_dir (src->client, GST_GCONF_DIR,
       GCONF_CLIENT_PRELOAD_RECURSIVE, NULL);
-  gconf_client_notify_add (src->client, 
+  gconf_client_notify_add (src->client,
       GST_GCONF_DIR "/" GST_GCONF_AUDIOSRC_KEY,
       cb_toggle_element, src, NULL, NULL);
 }
@@ -117,7 +116,7 @@ gst_gconf_audio_src_dispose (GObject * object)
     g_object_unref (G_OBJECT (src->client));
     src->client = NULL;
   }
-  
+
   g_free (src->gconf_str);
   src->gconf_str = NULL;
 
@@ -133,7 +132,7 @@ do_toggle_element (GstGConfAudioSrc * src)
   new_gconf_str = gst_gconf_get_string (GST_GCONF_AUDIOSRC_KEY);
   if (new_gconf_str != NULL && src->gconf_str != NULL &&
       (strlen (new_gconf_str) == 0 ||
-      strcmp (src->gconf_str, new_gconf_str) == 0)) {
+          strcmp (src->gconf_str, new_gconf_str) == 0)) {
     g_free (new_gconf_str);
     GST_DEBUG_OBJECT (src, "GConf key was updated, but it didn't change");
     return TRUE;
