@@ -28,12 +28,11 @@
 #include "gst/gst-i18n-plugin.h"
 #include <jerror.h>
 
-GstElementDetails gst_jpeg_dec_details = {
-  "JPEG image decoder",
-  "Codec/Decoder/Image",
-  "Decode images from JPEG format",
-  "Wim Taymans <wim.taymans@tvd.be>",
-};
+static const GstElementDetails gst_jpeg_dec_details =
+GST_ELEMENT_DETAILS ("JPEG image decoder",
+    "Codec/Decoder/Image",
+    "Decode images from JPEG format",
+    "Wim Taymans <wim.taymans@tvd.be>");
 
 #define MIN_WIDTH  16
 #define MAX_WIDTH  4096
@@ -57,7 +56,7 @@ GST_STATIC_PAD_TEMPLATE ("sink",
         G_STRINGIFY (MAX_HEIGHT) " ], " "framerate = (fraction) [ 0/1, MAX ]")
     );
 
-GST_DEBUG_CATEGORY (jpeg_dec_debug);
+GST_DEBUG_CATEGORY_STATIC (jpeg_dec_debug);
 #define GST_CAT_DEFAULT jpeg_dec_debug
 
 /* These macros are adapted from videotestsrc.c 
@@ -144,7 +143,9 @@ gst_jpeg_dec_class_init (GstJpegDecClass * klass)
   parent_class = g_type_class_peek_parent (klass);
 
   gobject_class->finalize = gst_jpeg_dec_finalize;
-  gstelement_class->change_state = gst_jpeg_dec_change_state;
+
+  gstelement_class->change_state =
+      GST_DEBUG_FUNCPTR (gst_jpeg_dec_change_state);
 
   GST_DEBUG_CATEGORY_INIT (jpeg_dec_debug, "jpegdec", 0, "JPEG decoder");
 }
@@ -230,8 +231,8 @@ gst_jpeg_dec_init (GstJpegDec * dec)
 
   /* create the sink and src pads */
   dec->sinkpad =
-      gst_pad_new_from_template (gst_static_pad_template_get
-      (&gst_jpeg_dec_sink_pad_template), "sink");
+      gst_pad_new_from_static_template (&gst_jpeg_dec_sink_pad_template,
+      "sink");
   gst_element_add_pad (GST_ELEMENT (dec), dec->sinkpad);
   gst_pad_set_setcaps_function (dec->sinkpad,
       GST_DEBUG_FUNCPTR (gst_jpeg_dec_setcaps));
@@ -239,8 +240,7 @@ gst_jpeg_dec_init (GstJpegDec * dec)
       GST_DEBUG_FUNCPTR (gst_jpeg_dec_chain));
 
   dec->srcpad =
-      gst_pad_new_from_template (gst_static_pad_template_get
-      (&gst_jpeg_dec_src_pad_template), "src");
+      gst_pad_new_from_static_template (&gst_jpeg_dec_src_pad_template, "src");
   gst_element_add_pad (GST_ELEMENT (dec), dec->srcpad);
 
   /* setup jpeglib */
@@ -604,6 +604,9 @@ gst_jpeg_dec_setcaps (GstPad * pad, GstCaps * caps)
   /* do not extract width/height here. we do that in the chain
    * function on a per-frame basis (including the line[] array
    * setup) */
+
+  /* But we can take the framerate values and set them on the src pad */
+
   return TRUE;
 }
 
@@ -655,12 +658,9 @@ gst_jpeg_dec_decode_direct (GstJpegDec * dec, guchar * base[3],
   guchar **line[3];             /* the jpeg line buffer */
   gint i, j, k;
 
-  line[0] = g_alloca ((r_v * DCTSIZE) * sizeof (guchar *));
-  line[1] = g_alloca ((r_v * DCTSIZE) * sizeof (guchar *));
-  line[2] = g_alloca ((r_v * DCTSIZE) * sizeof (guchar *));
-  memset (line[0], 0, (r_v * DCTSIZE) * sizeof (guchar *));
-  memset (line[1], 0, (r_v * DCTSIZE) * sizeof (guchar *));
-  memset (line[2], 0, (r_v * DCTSIZE) * sizeof (guchar *));
+  line[0] = g_new0 (guchar *, (r_v * DCTSIZE));
+  line[1] = g_new0 (guchar *, (r_v * DCTSIZE));
+  line[2] = g_new0 (guchar *, (r_v * DCTSIZE));
 
   /* let jpeglib decode directly into our final buffer */
   GST_DEBUG_OBJECT (dec, "decoding directly into output buffer");
@@ -685,6 +685,10 @@ gst_jpeg_dec_decode_direct (GstJpegDec * dec, guchar * base[3],
     }
     jpeg_read_raw_data (&dec->cinfo, line, r_v * DCTSIZE);
   }
+
+  g_free (line[0]);
+  g_free (line[1]);
+  g_free (line[2]);
 }
 
 
@@ -707,6 +711,13 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
   dec = GST_JPEG_DEC (GST_OBJECT_PARENT (pad));
 
   timestamp = GST_BUFFER_TIMESTAMP (buf);
+  duration = GST_BUFFER_DURATION (buf);
+
+/*
+  GST_LOG_OBJECT (dec, "Received buffer: %d bytes, ts=%" GST_TIME_FORMAT
+      ", dur=%" GST_TIME_FORMAT, GST_BUFFER_SIZE (buf),
+      GST_TIME_ARGS (timestamp), GST_TIME_ARGS (duration));
+*/
 
   if (GST_CLOCK_TIME_IS_VALID (timestamp)) {
     dec->next_ts = timestamp;
@@ -806,6 +817,7 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
     GST_DEBUG_OBJECT (dec, "max_v_samp_factor=%d",
         dec->cinfo.max_v_samp_factor);
 
+    gst_pad_use_fixed_caps (dec->srcpad);
     gst_pad_set_caps (dec->srcpad, caps);
     gst_caps_unref (caps);
 
@@ -823,15 +835,18 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
   outdata = GST_BUFFER_DATA (outbuf);
   GST_BUFFER_TIMESTAMP (outbuf) = dec->next_ts;
 
-  if (dec->packetized && dec->framerate_numerator != 0) {
-    GstClockTime next_ts;
-
-    dec->frames_decoded++;
-    next_ts = dec->frames_decoded * GST_SECOND *
-        dec->framerate_denominator / dec->framerate_numerator;
-
-    duration = next_ts - dec->next_ts;
-    dec->next_ts = next_ts;
+  if (dec->packetized && GST_CLOCK_TIME_IS_VALID (dec->next_ts)) {
+    if (GST_CLOCK_TIME_IS_VALID (duration)) {
+      /* use duration from incoming buffer for outgoing buffer */
+      dec->next_ts += duration;
+    } else if (dec->framerate_numerator != 0) {
+      duration = gst_util_uint64_scale (GST_SECOND,
+          dec->framerate_denominator, dec->framerate_numerator);
+      dec->next_ts += duration;
+    } else {
+      duration = GST_CLOCK_TIME_NONE;
+      dec->next_ts = GST_CLOCK_TIME_NONE;
+    }
   } else {
     duration = GST_CLOCK_TIME_NONE;
     dec->next_ts = GST_CLOCK_TIME_NONE;
@@ -864,7 +879,7 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
    * copy over the data into our final picture buffer, otherwise jpeglib might
    * write over the end of a line into the beginning of the next line,
    * resulting in blocky artifacts on the left side of the picture. */
-  if ((I420_Y_ROWSTRIDE (width) % (dec->cinfo.max_h_samp_factor * DCTSIZE)) > 0) {
+  if (width % (dec->cinfo.max_h_samp_factor * DCTSIZE) != 0) {
     gst_jpeg_dec_decode_indirect (dec, base, last, width, height, r_v);
   } else {
     gst_jpeg_dec_decode_direct (dec, base, last, width, height, r_v);
@@ -873,7 +888,10 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
   GST_LOG_OBJECT (dec, "decompressing finished");
   jpeg_finish_decompress (&dec->cinfo);
 
-  GST_LOG_OBJECT (dec, "pushing buffer");
+  GST_LOG_OBJECT (dec, "pushing buffer (ts=%" GST_TIME_FORMAT ", dur=%"
+      GST_TIME_FORMAT, GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (outbuf)),
+      GST_TIME_ARGS (GST_BUFFER_DURATION (outbuf)));
+
   ret = gst_pad_push (dec->srcpad, outbuf);
 
 done:
@@ -941,7 +959,6 @@ gst_jpeg_dec_change_state (GstElement * element, GstStateChange transition)
       dec->framerate_numerator = 0;
       dec->framerate_denominator = 1;
       dec->caps_framerate_numerator = dec->caps_framerate_denominator = 0;
-      dec->frames_decoded = 0;
       dec->caps_width = -1;
       dec->caps_height = -1;
       dec->packetized = FALSE;
@@ -951,6 +968,8 @@ gst_jpeg_dec_change_state (GstElement * element, GstStateChange transition)
   }
 
   ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
+  if (ret != GST_STATE_CHANGE_SUCCESS)
+    return ret;
 
   switch (transition) {
     case GST_STATE_CHANGE_PAUSED_TO_READY:
