@@ -27,12 +27,11 @@
 #include <gst/video/video.h>
 
 /* elementfactory information */
-GstElementDetails gst_jpegenc_details = {
-  "JPEG image encoder",
-  "Codec/Encoder/Image",
-  "Encode images in JPEG format",
-  "Wim Taymans <wim.taymans@tvd.be>",
-};
+static const GstElementDetails gst_jpegenc_details =
+GST_ELEMENT_DETAILS ("JPEG image encoder",
+    "Codec/Encoder/Image",
+    "Encode images in JPEG format",
+    "Wim Taymans <wim.taymans@tvd.be>");
 
 GST_DEBUG_CATEGORY (jpegenc_debug);
 #define GST_CAT_DEFAULT jpegenc_debug
@@ -152,7 +151,7 @@ gst_jpegenc_class_init (GstJpegEnc * klass)
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
-  parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
+  parent_class = g_type_class_peek_parent (klass);
 
   gst_jpegenc_signals[FRAME_ENCODED] =
       g_signal_new ("frame-encoded", G_TYPE_FROM_CLASS (klass),
@@ -205,17 +204,19 @@ gst_jpegenc_init (GstJpegEnc * jpegenc)
 {
   /* create the sink and src pads */
   jpegenc->sinkpad =
-      gst_pad_new_from_template (gst_static_pad_template_get
-      (&gst_jpegenc_sink_pad_template), "sink");
-  gst_pad_set_chain_function (jpegenc->sinkpad, gst_jpegenc_chain);
-  gst_pad_set_getcaps_function (jpegenc->sinkpad, gst_jpegenc_getcaps);
-  gst_pad_set_setcaps_function (jpegenc->sinkpad, gst_jpegenc_setcaps);
+      gst_pad_new_from_static_template (&gst_jpegenc_sink_pad_template, "sink");
+  gst_pad_set_chain_function (jpegenc->sinkpad,
+      GST_DEBUG_FUNCPTR (gst_jpegenc_chain));
+  gst_pad_set_getcaps_function (jpegenc->sinkpad,
+      GST_DEBUG_FUNCPTR (gst_jpegenc_getcaps));
+  gst_pad_set_setcaps_function (jpegenc->sinkpad,
+      GST_DEBUG_FUNCPTR (gst_jpegenc_setcaps));
   gst_element_add_pad (GST_ELEMENT (jpegenc), jpegenc->sinkpad);
 
   jpegenc->srcpad =
-      gst_pad_new_from_template (gst_static_pad_template_get
-      (&gst_jpegenc_src_pad_template), "src");
-  gst_pad_set_getcaps_function (jpegenc->sinkpad, gst_jpegenc_getcaps);
+      gst_pad_new_from_static_template (&gst_jpegenc_src_pad_template, "src");
+  gst_pad_set_getcaps_function (jpegenc->sinkpad,
+      GST_DEBUG_FUNCPTR (gst_jpegenc_getcaps));
   /*gst_pad_set_setcaps_function (jpegenc->sinkpad, gst_jpegenc_setcaps); */
   gst_pad_use_fixed_caps (jpegenc->sinkpad);
   gst_element_add_pad (GST_ELEMENT (jpegenc), jpegenc->srcpad);
@@ -280,6 +281,7 @@ gst_jpegenc_getcaps (GstPad * pad)
           GST_STR_FOURCC ("I420"), NULL);
     }
   }
+  gst_object_unref (jpegenc);
 
   return caps;
 }
@@ -322,6 +324,8 @@ gst_jpegenc_setcaps (GstPad * pad, GstCaps * caps)
     gst_jpegenc_resync (jpegenc);
   }
 
+  gst_object_unref (jpegenc);
+
   return ret;
 }
 
@@ -401,7 +405,7 @@ gst_jpegenc_chain (GstPad * pad, GstBuffer * buf)
   gulong size;
   GstBuffer *outbuf;
   guint height, width;
-  guchar *base[3];
+  guchar *base[3], *end[3];
   gint i, j, k;
 
   jpegenc = GST_JPEGENC (GST_OBJECT_PARENT (pad));
@@ -428,6 +432,10 @@ gst_jpegenc_chain (GstPad * pad, GstBuffer * buf)
   base[1] = data + I420_U_OFFSET (width, height);
   base[2] = data + I420_V_OFFSET (width, height);
 
+  end[0] = base[0] + height * I420_Y_ROWSTRIDE (width);
+  end[1] = base[1] + (height / 2) * I420_U_ROWSTRIDE (width);
+  end[2] = base[2] + (height / 2) * I420_V_ROWSTRIDE (width);
+
   jpegenc->jdest.next_output_byte = GST_BUFFER_DATA (outbuf);
   jpegenc->jdest.free_in_buffer = GST_BUFFER_SIZE (outbuf);
 
@@ -441,13 +449,17 @@ gst_jpegenc_chain (GstPad * pad, GstBuffer * buf)
     /*g_print ("next scanline: %d\n", jpegenc->cinfo.next_scanline); */
     for (j = 0, k = 0; j < (2 * DCTSIZE); j += 2, k++) {
       jpegenc->line[0][j] = base[0];
-      base[0] += I420_Y_ROWSTRIDE (width);
+      if (base[0] + I420_Y_ROWSTRIDE (width) < end[0])
+        base[0] += I420_Y_ROWSTRIDE (width);
       jpegenc->line[0][j + 1] = base[0];
-      base[0] += I420_Y_ROWSTRIDE (width);
+      if (base[0] + I420_Y_ROWSTRIDE (width) < end[0])
+        base[0] += I420_Y_ROWSTRIDE (width);
       jpegenc->line[1][k] = base[1];
-      base[1] += I420_U_ROWSTRIDE (width);
+      if (base[1] + I420_U_ROWSTRIDE (width) < end[1])
+        base[1] += I420_U_ROWSTRIDE (width);
       jpegenc->line[2][k] = base[2];
-      base[2] += I420_V_ROWSTRIDE (width);
+      if (base[2] + I420_V_ROWSTRIDE (width) < end[2])
+        base[2] += I420_V_ROWSTRIDE (width);
     }
     jpeg_write_raw_data (&jpegenc->cinfo, jpegenc->line, 2 * DCTSIZE);
   }
