@@ -28,12 +28,11 @@
 #define MAX_HEIGHT              4096
 
 
-static GstElementDetails gst_pngenc_details = {
-  "PNG encoder",
-  "Codec/Encoder/Image",
-  "Encode a video frame to a .png image",
-  "Jeremy SIMON <jsimon13@yahoo.fr>",
-};
+static const GstElementDetails gst_pngenc_details =
+GST_ELEMENT_DETAILS ("PNG image encoder",
+    "Codec/Encoder/Image",
+    "Encode a video frame to a .png image",
+    "Jeremy SIMON <jsimon13@yahoo.fr>");
 
 GST_DEBUG_CATEGORY (pngenc_debug);
 #define GST_CAT_DEFAULT pngenc_debug
@@ -117,7 +116,7 @@ gst_pngenc_class_init (GstPngEncClass * klass)
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
-  parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
+  parent_class = g_type_class_peek_parent (klass);
 
   gobject_class->get_property = gst_pngenc_get_property;
   gobject_class->set_property = gst_pngenc_set_property;
@@ -176,6 +175,9 @@ gst_pngenc_setcaps (GstPad * pad, GstCaps * caps)
     gst_caps_unref (pcaps);
     gst_object_unref (opeer);
   }
+
+  gst_object_unref (pngenc);
+
   return ret;
 }
 
@@ -183,8 +185,8 @@ static void
 gst_pngenc_init (GstPngEnc * pngenc, GstPngEncClass * g_class)
 {
   /* sinkpad */
-  pngenc->sinkpad = gst_pad_new_from_template
-      (gst_static_pad_template_get (&pngenc_sink_template), "sink");
+  pngenc->sinkpad = gst_pad_new_from_static_template
+      (&pngenc_sink_template, "sink");
   gst_pad_set_chain_function (pngenc->sinkpad, gst_pngenc_chain);
   /*   gst_pad_set_link_function (pngenc->sinkpad, gst_pngenc_sinklink); */
   /*   gst_pad_set_getcaps_function (pngenc->sinkpad, gst_pngenc_sink_getcaps); */
@@ -192,8 +194,8 @@ gst_pngenc_init (GstPngEnc * pngenc, GstPngEncClass * g_class)
   gst_element_add_pad (GST_ELEMENT (pngenc), pngenc->sinkpad);
 
   /* srcpad */
-  pngenc->srcpad = gst_pad_new_from_template
-      (gst_static_pad_template_get (&pngenc_src_template), "src");
+  pngenc->srcpad = gst_pad_new_from_static_template
+      (&pngenc_src_template, "src");
   /*   pngenc->srcpad = gst_pad_new ("src", GST_PAD_SRC); */
   /*   gst_pad_set_getcaps_function (pngenc->srcpad, gst_pngenc_src_getcaps); */
   /*   gst_pad_set_setcaps_function (pngenc->srcpad, gst_pngenc_setcaps); */
