@@ -19,7 +19,7 @@
 #define STOP_SPEED 128
 
 void
-goom_init (GoomData *goomdata, guint32 resx, guint32 resy)
+goom_init (GoomData * goomdata, guint32 resx, guint32 resy)
 {
 #ifdef VERBOSE
   printf ("GOOM: init (%d, %d);\n", resx, resy);
@@ -38,27 +38,26 @@ goom_init (GoomData *goomdata, guint32 resx, guint32 resy)
   goomdata->cycle = 0;
 
 
-  goomdata->goomlimit = 2;    /* sensibilit du goom */
-  goomdata->zfd = zoomFilterNew();
-  goomdata->lockvar = 0;       /* pour empecher de nouveaux changements */
-  goomdata->goomvar = 0;       /* boucle des gooms */
-  goomdata->totalgoom = 0;     /* nombre de gooms par seconds */
-  goomdata->agoom = 0;         /* un goom a eu lieu..       */
-  goomdata->loopvar = 0;       /* mouvement des points */
-  goomdata->speedvar = 0;      /* vitesse des particules */
-  goomdata->lineMode = 0;      /* l'effet lineaire a dessiner */
+  goomdata->goomlimit = 2;      /* sensibilit du goom */
+  goomdata->zfd = zoomFilterNew ();
+  goomdata->lockvar = 0;        /* pour empecher de nouveaux changements */
+  goomdata->goomvar = 0;        /* boucle des gooms */
+  goomdata->totalgoom = 0;      /* nombre de gooms par seconds */
+  goomdata->agoom = 0;          /* un goom a eu lieu..       */
+  goomdata->loopvar = 0;        /* mouvement des points */
+  goomdata->speedvar = 0;       /* vitesse des particules */
+  goomdata->lineMode = 0;       /* l'effet lineaire a dessiner */
 }
 
 void
-goom_set_resolution (GoomData *goomdata, guint32 resx, guint32 resy)
+goom_set_resolution (GoomData * goomdata, guint32 resx, guint32 resy)
 {
   guint32 buffsize = resx * resy;
 
   if ((goomdata->resolx == resx) && (goomdata->resoly == resy))
     return;
 
-  if (goomdata->buffsize < buffsize)
-  {
+  if (goomdata->buffsize < buffsize) {
     if (goomdata->pixel)
       free (goomdata->pixel);
     if (goomdata->back)
@@ -67,7 +66,8 @@ goom_set_resolution (GoomData *goomdata, guint32 resx, guint32 resy)
     goomdata->back = (guint32 *) malloc (buffsize * sizeof (guint32) + 128);
     goomdata->buffsize = buffsize;
 
-    goomdata->p1 = (void *) (((unsigned long) goomdata->pixel + 0x7f) & (~0x7f));
+    goomdata->p1 =
+        (void *) (((unsigned long) goomdata->pixel + 0x7f) & (~0x7f));
     goomdata->p2 = (void *) (((unsigned long) goomdata->back + 0x7f) & (~0x7f));
   }
 
@@ -79,7 +79,7 @@ goom_set_resolution (GoomData *goomdata, guint32 resx, guint32 resy)
 }
 
 guint32 *
-goom_update (GoomData *goomdata, gint16 data[2][512])
+goom_update (GoomData * goomdata, gint16 data[2][512])
 {
   guint32 *return_val;
   guint32 pointWidth;
@@ -125,7 +125,8 @@ goom_update (GoomData *goomdata, gint16 data[2][512])
 
   /* ! calcul du deplacement des petits points ... */
 
-  largfactor = ((float) goomdata->speedvar / 40.0f + (float) incvar / 50000.0f) / 1.5f;
+  largfactor =
+      ((float) goomdata->speedvar / 40.0f + (float) incvar / 50000.0f) / 1.5f;
   if (largfactor > 1.5f)
     largfactor = 1.5f;
 
@@ -169,8 +170,8 @@ goom_update (GoomData *goomdata, gint16 data[2][512])
   if ((accelvar > goomdata->goomlimit) || (accelvar < -goomdata->goomlimit)) {
     /* UN GOOM !!! YAHOO ! */
     goomdata->totalgoom++;
-    goomdata->agoom = 20;                 /* mais pdt 20 cycles, il n'y en aura plus. */
-    goomdata->lineMode = (goomdata->lineMode + 1) % 20;     /* Tous les 10 gooms on change de mode lineaire */
+    goomdata->agoom = 20;       /* mais pdt 20 cycles, il n'y en aura plus. */
+    goomdata->lineMode = (goomdata->lineMode + 1) % 20; /* Tous les 10 gooms on change de mode lineaire */
 
     /* changement eventuel de mode */
     switch (iRAND (goomdata, 10)) {
@@ -302,7 +303,7 @@ goom_update (GoomData *goomdata, gint16 data[2][512])
           pzfd->hPlaneEffect = iRAND (goomdata, 2) ? 0 : pzfd->hPlaneEffect;
         }
 
-        if (newvit < pzfd->vitesse) {     /* on accelere */
+        if (newvit < pzfd->vitesse) {   /* on accelere */
           zfd_update = 1;
           if (((newvit < STOP_SPEED - 7) &&
                   (pzfd->vitesse < STOP_SPEED - 6) &&
@@ -331,7 +332,8 @@ goom_update (GoomData *goomdata, gint16 data[2][512])
   }
 
   /* gros frein si la musique est calme */
-  if ((goomdata->speedvar < 1) && (pzfd->vitesse < STOP_SPEED - 4) && (goomdata->cycle % 16 == 0)) {
+  if ((goomdata->speedvar < 1) && (pzfd->vitesse < STOP_SPEED - 4)
+      && (goomdata->cycle % 16 == 0)) {
     /*
        printf ("++slow part... %i\n", zfd.vitesse) ;
      */
@@ -367,7 +369,8 @@ goom_update (GoomData *goomdata, gint16 data[2][512])
   /* si on est dans un goom : afficher les lignes... */
   if (goomdata->agoom > 15)
     goom_lines
-        (goomdata, data, ((pzfd->middleX == resolx / 2) && (pzfd->middleY == resoly / 2)
+        (goomdata, data, ((pzfd->middleX == resolx / 2)
+            && (pzfd->middleY == resoly / 2)
             && (pzfd->mode != WATER_MODE))
         ? (goomdata->lineMode / 10) : 0, goomdata->p2, goomdata->agoom - 15);
 
@@ -395,12 +398,16 @@ goom_update (GoomData *goomdata, gint16 data[2][512])
 }
 
 void
-goom_close (GoomData *goomdata)
+goom_close (GoomData * goomdata)
 {
   if (goomdata->pixel != NULL)
     free (goomdata->pixel);
   if (goomdata->back != NULL)
     free (goomdata->back);
+  if (goomdata->zfd != NULL) {
+    zoomFilterDestroy (goomdata->zfd);
+    goomdata->zfd = NULL;
+  }
   goomdata->pixel = goomdata->back = NULL;
   RAND_CLOSE (goomdata);
 }
