@@ -22,13 +22,15 @@
 
 #include "gstrtpmp4vpay.h"
 
+GST_DEBUG_CATEGORY (rtpmp4vpay_debug);
+#define GST_CAT_DEFAULT (rtpmp4vpay_debug)
+
 /* elementfactory information */
-static GstElementDetails gst_rtp_mp4vpay_details = {
-  "RTP packet parser",
-  "Codec/Payloader/Network",
-  "Payode MPEG4 video as RTP packets (RFC 3016)",
-  "Wim Taymans <wim@fluendo.com>"
-};
+static const GstElementDetails gst_rtp_mp4vpay_details =
+GST_ELEMENT_DETAILS ("RTP packet parser",
+    "Codec/Payloader/Network",
+    "Payode MPEG4 video as RTP packets (RFC 3016)",
+    "Wim Taymans <wim@fluendo.com>");
 
 static GstStaticPadTemplate gst_rtp_mp4v_pay_sink_template =
 GST_STATIC_PAD_TEMPLATE ("sink",
@@ -129,7 +131,7 @@ gst_rtp_mp4v_pay_class_init (GstRtpMP4VPayClass * klass)
   gstelement_class = (GstElementClass *) klass;
   gstbasertppayload_class = (GstBaseRTPPayloadClass *) klass;
 
-  parent_class = g_type_class_ref (GST_TYPE_BASE_RTP_PAYLOAD);
+  parent_class = g_type_class_peek_parent (klass);
 
   gobject_class->set_property = gst_rtp_mp4v_pay_set_property;
   gobject_class->get_property = gst_rtp_mp4v_pay_get_property;
@@ -143,6 +145,10 @@ gst_rtp_mp4v_pay_class_init (GstRtpMP4VPayClass * klass)
 
   gstbasertppayload_class->set_caps = gst_rtp_mp4v_pay_setcaps;
   gstbasertppayload_class->handle_buffer = gst_rtp_mp4v_pay_handle_buffer;
+
+  GST_DEBUG_CATEGORY_INIT (rtpmp4vpay_debug, "rtpmp4vpay", 0,
+      "MP4 video RTP Payloader");
+
 }
 
 static void
@@ -192,12 +198,43 @@ static gboolean
 gst_rtp_mp4v_pay_setcaps (GstBaseRTPPayload * payload, GstCaps * caps)
 {
   GstRtpMP4VPay *rtpmp4vpay;
+  GstStructure *structure;
+  const GValue *codec_data;
 
   rtpmp4vpay = GST_RTP_MP4V_PAY (payload);
 
   gst_basertppayload_set_options (payload, "video", TRUE, "MP4V-ES",
       rtpmp4vpay->rate);
 
+  structure = gst_caps_get_structure (caps, 0);
+  codec_data = gst_structure_get_value (structure, "codec_data");
+  if (codec_data) {
+    GST_LOG_OBJECT (rtpmp4vpay, "got codec_data");
+    if (G_VALUE_TYPE (codec_data) == GST_TYPE_BUFFER) {
+      GstBuffer *buffer;
+      guint8 *data;
+      guint size;
+
+      buffer = gst_value_get_buffer (codec_data);
+
+      data = GST_BUFFER_DATA (buffer);
+      size = GST_BUFFER_SIZE (buffer);
+
+      if (size < 5)
+        goto done;
+
+      rtpmp4vpay->profile = data[4];
+      GST_LOG_OBJECT (rtpmp4vpay, "configuring codec_data, profile %d",
+          data[4]);
+
+      if (rtpmp4vpay->config)
+        gst_buffer_unref (rtpmp4vpay->config);
+      rtpmp4vpay->config = gst_buffer_copy (buffer);
+      gst_rtp_mp4v_pay_new_caps (rtpmp4vpay);
+    }
+  }
+
+done:
   return TRUE;
 }
 
@@ -275,6 +312,7 @@ gst_rtp_mp4v_pay_depay_data (GstRtpMP4VPay * enc, guint8 * data, guint size,
     return FALSE;
 
   code = GST_READ_UINT32_BE (data);
+  GST_DEBUG_OBJECT (enc, "start code 0x%08x", code);
 
   switch (code) {
     case VOS_STARTCODE:
@@ -287,6 +325,8 @@ gst_rtp_mp4v_pay_depay_data (GstRtpMP4VPay * enc, guint8 * data, guint size,
       /* profile_and_level_indication */
       profile = data[4];
 
+      GST_DEBUG_OBJECT (enc, "VOS profile 0x%08x", profile);
+
       if (profile != enc->profile) {
         newprofile = TRUE;
         enc->profile = profile;
@@ -322,10 +362,12 @@ gst_rtp_mp4v_pay_depay_data (GstRtpMP4VPay * enc, guint8 * data, guint size,
       break;
     }
     case VOP_STARTCODE:
+      GST_DEBUG_OBJECT (enc, "VOP");
       /* VOP startcode, we don't have to flush the packet */
       result = FALSE;
       break;
     default:
+      GST_DEBUG_OBJECT (enc, "other startcode");
       /* all other startcodes need a flush */
       result = TRUE;
       break;
